precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform lowp int isPreview;
uniform lowp sampler2D s_textureMapping;
uniform float u_offset;
uniform vec2 texSize;
uniform vec2 asciiTexSize;

const vec3  RGB2GRAY_VEC3 = vec3(0.299, 0.587, 0.114);
const float MESH_WIDTH = 16.0;
const float MESH_HEIGHT= 23.0;
const float GARY_LEVEL = 24.0;
const float ASCIIS_WIDTH = 8.0;
const float ASCIIS_HEIGHT = 3.0;
const float MESH_ROW_NUM = 100.0;
void main()
{
    float imageMeshWidth = texSize.x / MESH_ROW_NUM;
    float imageMeshHeight = imageMeshWidth * MESH_HEIGHT / MESH_WIDTH;

    vec2 imageTexCoord = textureCoordinate * texSize;
    vec2 midTexCoord;
    midTexCoord.x = floor(imageTexCoord.x / imageMeshWidth) * imageMeshWidth + imageMeshWidth * 0.5;
    midTexCoord.y = floor(imageTexCoord.y / imageMeshHeight) * imageMeshHeight + imageMeshHeight * 0.5;

    vec2 normalizedTexCoord = midTexCoord / texSize;
    vec4 rgbColor = texture2D(inputImageTexture, normalizedTexCoord);
    float grayValue = dot(rgbColor.rgb, RGB2GRAY_VEC3);
    //gl_FragColor = vec4(vec3(grayValue), rgbColor.a);

    float offsetX = mod(imageTexCoord.x, imageMeshWidth) * MESH_WIDTH / imageMeshWidth;
    float offsetY = mod(imageTexCoord.y, imageMeshHeight) * MESH_HEIGHT / imageMeshHeight;

    float asciiIndex = floor((1.0 - grayValue) * GARY_LEVEL);
    float asciiIndexX = mod(asciiIndex, ASCIIS_WIDTH);
    float asciiIndexY = floor(asciiIndex / ASCIIS_WIDTH);

    vec2 grayTexCoord;
    grayTexCoord.x = (asciiIndexX * MESH_WIDTH + offsetX) / asciiTexSize.x;
    grayTexCoord.y = (asciiIndexY * MESH_HEIGHT + offsetY) / asciiTexSize.y;

    vec4 originColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 mappingColor = vec4(texture2D(s_textureMapping, grayTexCoord).rgb, rgbColor.a);

    gl_FragColor = mix(originColor, mappingColor, u_offset);
}